package com.stormtales.notevault.ui.songeditor;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.text.Layout;
import android.util.Log;
import android.widget.Button;
import android.widget.EditText;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import androidx.lifecycle.ViewModelProvider;
import com.stormtales.notevault.R;
import com.stormtales.notevault.data.entities.NoteSheet;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.sync.SyncStatus;
import com.stormtales.notevault.network.sync.SongSyncService;
import com.stormtales.notevault.network.sync.models.AIRecognizedSong;
import com.stormtales.notevault.ui.home.HomeViewModel;
import com.stormtales.notevault.utils.NoteSheetsUtil;
import com.stormtales.notevault.utils.Tupel;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;


public class SongEditorDialog extends DialogFragment {

    Dialog dialog;
    private Uri[] noteSheetFiles;
    private HomeViewModel homeViewModel;

    private Song editedSong;
    private SongSyncService songSyncService;

    private View progressbar;
    public SongEditorDialog() {
        // Required empty public constructor
        this.songSyncService = new SongSyncService(this.getContext());
    }


    @NonNull
    @Override
    public @NotNull Dialog onCreateDialog(@Nullable @org.jetbrains.annotations.Nullable Bundle savedInstanceState) {
        LayoutInflater inflater = LayoutInflater.from(getContext());
        View dialogView = inflater.inflate(R.layout.fragment_song_editor_dialog, null);


        dialogView.findViewById(R.id.btnCancel).setOnClickListener(v-> onCancel());
        dialogView.findViewById(R.id.btnSave).setOnClickListener(v -> onSave());

        if(this.noteSheetFiles == null || noteSheetFiles.length == 0) {
            dialogView.findViewById(R.id.btnAutoDetect).setEnabled(false);
        } else {
            progressbar = dialogView.findViewById(R.id.spinnerAutoDetect);

        }

        dialogView.findViewById(R.id.btnAutoDetect).setOnClickListener(v -> {
            progressbar.setVisibility(ViewGroup.VISIBLE);
            extractTitleFromFirstPage();
        });

        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setCancelable(false);
        builder.setView(dialogView);

        dialog = builder.create();
        dialog.setCanceledOnTouchOutside(false);
        return dialog;
    }

    private File createTemporaryFileForRecognition(Uri noteSheetUri) {
        try {
            // Hole den InputStream des Bildes
            InputStream inputStream = getContext().getContentResolver().openInputStream(noteSheetUri);
            if (inputStream != null) {
                // Erstelle eine temporäre Datei
                File tempFile = File.createTempFile("temp_note_sheet", ".jpg", getContext().getCacheDir());

                // Inhalt des InputStreams in die temporäre Datei schreiben
                try (OutputStream outputStream = new FileOutputStream(tempFile)) {
                    byte[] buffer = new byte[8192];
                    int bytesRead;
                    while ((bytesRead = inputStream.read(buffer)) != -1) {
                        outputStream.write(buffer, 0, bytesRead);
                    }
                }

                // Schließe den InputStream
                inputStream.close();

                return tempFile;
            }
        } catch (Exception e) {
            Log.e("SongEditorDialog", "Fehler beim Erstellen der temporären Datei: " + e.getMessage());
        }

        return null;
    }
    private void extractTitleFromFirstPage() {
        if(noteSheetFiles != null && noteSheetFiles.length > 0) {
            Uri firstPageUri = noteSheetFiles[0];

            // Temporäre Datei erstellen
            File tempFile = createTemporaryFileForRecognition(firstPageUri);
            if(tempFile != null) {
                songSyncService.recognizeTitle(tempFile.getAbsolutePath(), new SongSyncService.RecognizedSongCallback() {
                    @Override
                    public void onRecognizedSong(AIRecognizedSong aiRecognizedSong) {
                        progressbar.setVisibility(ViewGroup.GONE);
                        ((EditText) dialog.findViewById(R.id.etTitle)).setText(aiRecognizedSong.getTitle());
                        ((EditText) dialog.findViewById(R.id.etComposer)).setText(aiRecognizedSong.getComposer());
                        ((EditText) dialog.findViewById(R.id.etYear)).setText(String.valueOf(aiRecognizedSong.getYear()));
                        ((EditText) dialog.findViewById(R.id.etGenre)).setText(aiRecognizedSong.getDescription());
                    }
                });
            }
        }
    }

    @Override
    public void onStart() {
        super.onStart();
        if(editedSong != null) {
            ((EditText) dialog.findViewById(R.id.etTitle)).setText(editedSong.getTitle());
            ((EditText) dialog.findViewById(R.id.etComposer)).setText(editedSong.getComposer());
            ((EditText) dialog.findViewById(R.id.etGenre)).setText(editedSong.getGenre());
            ((EditText) dialog.findViewById(R.id.etYear)).setText(String.valueOf(editedSong.getYear()));
        }
    }

    private void onCancel() {
        dialog.dismiss();
    }

    private void onSave() {
        String title = ((EditText) dialog.findViewById(R.id.etTitle)).getText().toString();
        String composer = ((EditText) dialog.findViewById(R.id.etComposer)).getText().toString();

        String year_string = ((EditText) dialog.findViewById(R.id.etYear)).getText().toString();
        int releaseYear = year_string.isBlank()? 0 : Integer.parseInt(year_string);
        String genre = ((EditText) dialog.findViewById(R.id.etGenre)).getText().toString();

        if(editedSong != null) {
            editedSong.setTitle(title);
            editedSong.setComposer(composer);
            editedSong.setYear(releaseYear);
            editedSong.setGenre(genre);
            editedSong.setSyncStatus(SyncStatus.MODIFIED);
            homeViewModel.updateSong(editedSong);
        } else {
            Song song = new Song(title, composer, genre, releaseYear);

            Context context = this.getContext();
            List<NoteSheet> noteSheetList = new ArrayList<>();
            for(Uri uri : noteSheetFiles) {
                Tupel<String, String> result = NoteSheetsUtil.saveImageInternally(context.getContentResolver(), uri, context.getFilesDir());
                noteSheetList.add(new NoteSheet(result.getValue00(), result.getValue01()));
            }
            homeViewModel.addSong(song, noteSheetList);
        }


        dialog.dismiss();

    }

    public void setNoteSheetFiles(Uri[] noteSheetFiles) {
        this.noteSheetFiles = noteSheetFiles;
    }

    public void setHomeViewModel(HomeViewModel homeViewModel) {
        this.homeViewModel = homeViewModel;
    }

    public void setEditedSong(Song editedSong) {
        this.editedSong = editedSong;
    }
}